/*
 * map mysql_field_count() to mysql_num_fields()
 * if the former isn't available
 */

#if !defined(MYSQL_VERSION_ID) || (MYSQL_VERSION_ID<32224)
#define mysql_field_count mysql_num_fields
#endif

/* #@ _PROCESS_QUERY_ */
void
process_query (MYSQL *conn, char *query)
{
MYSQL_RES *res_set;

	if (mysql_query (conn, query) != 0)	/* the query failed */
	{
		print_error (conn, "Could not execute query");
		return;
	}

	/* the query succeeded; determine whether or not it returns data */

	res_set = mysql_store_result (conn);
	if (res_set)			/* a result set was returned */
	{
		/* process rows, then free the result set */
		process_result_set (conn, res_set);
		mysql_free_result (res_set);
	}
	else					/* no result set was returned */
	{
		/*
		 * does the lack of a result set mean that the query didn't
		 * return one, or that it should have but an error occurred?
		 */
		if (mysql_field_count (conn) == 0)
		{
			/*
			 * query generated no result set (it was not a SELECT, SHOW,
			 * DESCRIBE, etc.), so just report number of rows affected
			 */
			printf ("%lu rows affected\n",
						(unsigned long) mysql_affected_rows (conn));
		}
		else	/* an error occurred */
		{
			print_error (conn, "Could not retrieve result set");
		}
	}
}
/* #@ _PROCESS_QUERY_ */
